from .allguild import *
from .events import *
from .fetch import *
from .funcs import *
from .guild import *
from .main import *
from .member import *
from .message import *
from .permissions import *
from .role import *
from .embed import *
from .file import *

#so the idiots who made py-right extended dont have to be harassed even more because of "could not find [random attrubute] in 'hummus'"

# but like seriously what the hell is "imports is un-sorted/formatted"!?!?!?!?